@@ -6,29 +6,28 @@
 #ifndef _FCITX_LIBIME_CORE_DICTIONARY_H_
 #define _FCITX_LIBIME_CORE_DICTIONARY_H_
 
-#include "lattice.h"
 #include "libimecore_export.h"
-#include "segmentgraph.h"
 #include <functional>
-#include <string_view>
+#include <libime/core/lattice.h>
+#include <libime/core/segmentgraph.h>
+#include <memory>
+#include <unordered_set>
 
 namespace libime {
 
 class WordNode;
 
 // The callback accepts the passed path that matches the word.
-typedef std::function<void(const SegmentGraphPath &, WordNode &, float,
-                           std::unique_ptr<LatticeNodeData>)>
-    GraphMatchCallback;
+using GraphMatchCallback =
+    std::function<void(const SegmentGraphPath &, WordNode &, float,
+                       std::unique_ptr<LatticeNodeData>)>;
 
 class LIBIMECORE_EXPORT Dictionary {
 public:
     void
     matchPrefix(const SegmentGraph &graph, const GraphMatchCallback &callback,
                 const std::unordered_set<const SegmentGraphNode *> &ignore = {},
-                void *helper = nullptr) const {
-        matchPrefixImpl(graph, callback, ignore, helper);
-    }
+                void *helper = nullptr) const;
 
 protected:
     virtual void
